<?php

namespace ThomasKieslich\Tkautocrop\Services;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Thomas Kieslich
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
use ThomasKieslich\Tkautocrop\Hooks\CropScaleHook;
use TYPO3\CMS\Core\Resource\FileReference;
use TYPO3\CMS\Core\Resource\ProcessedFile;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;

/**
 * Class Ttcontent
 *
 * @package ThomasKieslich\Tkcropthumbs\Services
 */
class Ttcontent extends CropScaleHook {

	/**
	 * @param $file
	 * @param $fileArray
	 * @param $imageResource
	 * @param ContentObjectRenderer $parent
	 * @return array|null
	 */
	public function init($file, $fileArray, $imageResource, ContentObjectRenderer $parent) {
		$cropData = array();
		if (MathUtility::canBeInterpretedAsInteger($file) && $fileArray['import.']
				&& $parent->data['tx_tkautocrop_aspectratio'] > 0
		) {
			$aspectRatio = GeneralUtility::trimExplode(':', $parent->data['tx_tkautocrop_aspectratio']);
			if (count($aspectRatio) === 2) {
				$cropData['aspectRatio'] = $aspectRatio;
			}

			$fileRepository = GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Resource\\FileRepository');
			/** @var  FileReference $fileReference */
			$fileReference = $fileRepository->findFileReferenceByUid($parent->getCurrentVal());
			$currentFileProperties = $fileReference->getProperties();

			if ($cropData) {
				$cropData['originalImage']['width'] = $currentFileProperties['width'];
				$cropData['originalImage']['height'] = $currentFileProperties['height'];
			}
		}

		if (empty($cropData)) {
			return NULL;
		}
		/** @var ProcessedFile $processedFile */
		$processedFile = $imageResource['processedFile'];
		$processingConfiguration = $processedFile->getProcessingConfiguration();
		$cropParameters = $this->calcCrop($cropData, $processingConfiguration);

		return $cropParameters;
	}

	/**
	 * calculate new processing values
	 *
	 * @param array $cropData
	 * @param array $processingConfiguration
	 * @return array mixed
	 */
	protected function calcCrop($cropData, $processingConfiguration) {
		$width = (int)$processingConfiguration['width'];
		$height = (int)$processingConfiguration['height'];

		$maxWidth = (int)$processingConfiguration['maxWidth'];
		$maxHeight = (int)$processingConfiguration['maxHeight'];

		if ($maxWidth && $width > $maxWidth) {
			$width = $maxWidth;
		}

		if ($maxHeight && $height > $maxHeight) {
			$height = $maxHeight;
		}

		$arValues = $cropData['aspectRatio'];

		if ($maxWidth && !$width && !$height) {
			$width = $maxWidth;
			$height = (int)$width * ($arValues[1] / $arValues[0]);
		} elseif ($maxWidth && $width) {
			$height = (int)$width * ($arValues[1] / $arValues[0]);
		} elseif ($height && $width) {
			$width = (int)$height * ($arValues[0] / $arValues[1]);
			if ($maxWidth && $maxWidth <= $width) {
				$width = $maxWidth;
				$height = (int)$width * ($arValues[1] / $arValues[0]);
			}
		}

		//set values
		$processingConfiguration['maxWidth'] = '';
		$processingConfiguration['maxHeight'] = '';
		$processingConfiguration['width'] = (int)$width . 'c';
		$processingConfiguration['height'] = (int)$height . 'c';

		return $processingConfiguration;
	}
}